'''
given the input JSON file of SegmentationCategoryTypeModifier type, output
a JSON that contains only codes that include 3dSlicerLabel attribute. While
doing this, make sure that attributes of the parent elements are preserved
when the codes for modifiers/types contain 3dSlicerLabel attribute

I am sure there is a better way to do this! -@fedorov

'''

import json, sys, csv, re

jFileIn = sys.argv[1]

jIn= json.loads(open(jFileIn).read())

# clean up Modifiers that don't have Slicer labels
for c in range(len(jIn["SegmentationCodes"]["Category"])):
  for t in range(len(jIn["SegmentationCodes"]["Category"][c]["Type"])):
    if "Modifier" in jIn["SegmentationCodes"]["Category"][c]["Type"][t].keys():
      origList = jIn["SegmentationCodes"]["Category"][c]["Type"][t]["Modifier"]
      newList = [i for i in origList if "3dSlicerLabel" in i.keys()]
      if len(newList):
        jIn["SegmentationCodes"]["Category"][c]["Type"][t]["Modifier"] = newList
      else:
        jIn["SegmentationCodes"]["Category"][c]["Type"][t].pop("Modifier")

# clean up types that don't have modifiers and don't have Slicer label
for c in range(len(jIn["SegmentationCodes"]["Category"])):
  origList = jIn["SegmentationCodes"]["Category"][c]["Type"]
  newList = [i for i in origList if (("3dSlicerLabel" in i.keys()) or ("Modifier" in i.keys()))]
  if len(newList):
    jIn["SegmentationCodes"]["Category"][c]["Type"] = newList
  else:
    jIn["SegmentationCodes"]["Category"][c].pop("Type")

# clean up empty categories
origList = jIn["SegmentationCodes"]["Category"]
newList = [i for i in origList if "Type" in i.keys()]
newList = [i for i in newList if len(i["Type"])]
if len(newList):
  jIn["SegmentationCodes"]["Category"] = newList
else:
  jIn["SegmentationCodes"].pop("Category")

print json.dumps(jIn,indent=2)



import glob
import os
from typing import List
from pathlib import Path

import pydicom

from .utils.dicom import is_file_a_dicom
from .utils.bin import make_exec_bin, run_bin
from .utils.fileops import rename_file

from .base import BaseConverter


class Dcm2Nii(BaseConverter):
    def __init__(self):
        make_exec_bin()
        self.run_bin = run_bin
        super().__init__()

    @staticmethod
    def get_all_dicoms(dicom_directory: Path) -> List[Path]:
        """Class method to read all dicoms in adirectory

        Args:
            dicom_directory (Path): the directory where

        Raises:
            NameError: when the directory does not contain a single dicom
            NameError: when the direcotry does not exist

        Returns:
            List[Path]: list of all dicom-paths in the directory
        """

        if not os.path.exists(dicom_directory):
            raise NameError(f"directory: '{dicom_directory}' not found!")

        # all the files in the directory
        file_path_list = [
            file_path
            for file_path in glob.glob(os.path.join(dicom_directory, "**"))
            if os.path.isfile(file_path)
        ]

        # check for dicoms only
        dicom_path_list = [
            Path(file_path)
            for file_path in file_path_list
            if is_file_a_dicom(file_path)
        ]

        if len(dicom_path_list) == 0:
            raise NameError("Incorrect Path. No Dicoms found!")
        return dicom_path_list

    @staticmethod
    def check_slice_thickness_variable(all_dcm_paths: List[Path]) -> bool:
        """read file header slice thickness to determine if uniform thickness or variable

        Args:
            all_dcm_paths (List[Path]): list of path to DICOMs

        Returns:
            bool: True if variable slice thickness else False
        """
        all_slice_thickness = set(
            [pydicom.read_file(path).SliceThickness for path in all_dcm_paths]
        )
        return False if len(all_slice_thickness) == 1 else True

    def _run_conv_variable(self, dicom_directory: Path, out_directory: Path) -> List[Path]:
        raise NotImplementedError(
            "DICOM with variable slice thickness cannot to be handled yet!!"
        )

    def rename_converted_files(
        self, inp_file_list: List[Path], name: str
    ) -> List[Path]:
        """Rename a file while preserving the suffix from dcm2niix

        Args:
            inp_file_list (List[Path]): list of files to renamed
            name (str): new name of the file

        Returns:
            List[Path]: list of renamed files
        """
        # preserve all suffixes
        dcm2niix_suffix = [
            os.path.basename(file_path).split("_")[1:] for file_path in inp_file_list
        ]

        out_file_list = []

        # rename files
        for i, file_path in enumerate(inp_file_list):
            # name + "_" + "_".join(dcm2niix_suffix[i])
            # if suffix exists only
            # fileName_ + sufix1_suffi2
            # fileName_sufix1_suffi2
            fname = (
                name + "_" + "_".join(dcm2niix_suffix[i])
                if len(dcm2niix_suffix[i]) > 0
                else name
            )
            out_file_list.append(rename_file(str(file_path), fname))
        return out_file_list

    def _run_conv_uniform(self, dicom_directory: Path, out_directory:Path) -> List[Path]:
        """run the binary on the input directory

        Args:
            dicom_directory (Path): directory of the dicoms

        Returns:
            List[Path]: output NifTi files post conversion
        """
        self.run_bin(dicom_directory, out_directory)
        if out_directory is not None:
            dicom_directory = out_directory
        output_files = list(Path(dicom_directory).glob("*.nii*"))
        return output_files

    def run(self, dicom_directory: Path, out_directory:Path=None, name: str = "") -> List[Path]:
        """Run the dcm to nifti conversion in a directory

        Args:
            dicom_directory (Path): path to directory with Dicoms
            dicom_directory (Path, optional): directory to store the nifti
            name (str, optional): Name to be given to the output file. Defaults to standard name.

        Raises:
            RuntimeError: Parsing dicom error
            RuntimeError: Conversion error
            RuntimeError: Renaming error

        Returns:
            List[Path]: output list of Nifti files
        """
        try:
            all_dcm_paths = self.get_all_dicoms(dicom_directory)
        except Exception as err:
            raise RuntimeError(f"Error parsing dicoms: {err}")

        try:
            if self.check_slice_thickness_variable(all_dcm_paths):
                converted_file_paths = self._run_conv_variable(dicom_directory, out_directory)
            else:
                converted_file_paths = self._run_conv_uniform(dicom_directory, out_directory)
        except Exception as err:
            raise RuntimeError(f"Error converting DCM to NifTi: {err}")

        if name != "":
            try:
                converted_file_paths = self.rename_converted_files(
                    converted_file_paths, name
                )
            except Exception as err:
                raise RuntimeError(f"Error renaming output NifTi: {err}")

        print(
            f"\nConverted {len(all_dcm_paths)} DCM to Nifti; Output stored @ {Path(converted_file_paths[0]).parent}\n"
        )

        return converted_file_paths



import os


def rename_file(complete_path: str, new_name: str) -> str:
    """rename a file with a new name

    Args:
        complete_path (str): full path to the old file
        new_name (str): new file name

    Raises:
        RuntimeError: Conversion failed

    Returns:
        str: full path to the new file
    """
    try:
        new_name_wo_ext = os.path.splitext(new_name)[0]
        file_name_wo_ext = os.path.splitext(os.path.basename(complete_path))[0]

        new_name_path = complete_path.replace(file_name_wo_ext, new_name_wo_ext)

        os.rename(complete_path, new_name_path)
    except Exception as err:
        raise RuntimeError(f"Unable to rename files: {err}")
    return new_name_path



import os
from pathlib import Path
from typing import List, Union

import numpy as np
import nibabel as nib
import pydicom
from pydicom.dataset import FileDataset, Dataset
import pydicom_seg
from pydicom_seg.segmentation_dataset import SegmentationDataset
import SimpleITK as sitk


from .base import BaseConverter
from .utils.json_helpers import verify_label_dcmqii_json


class Nii2DcmSeg(BaseConverter):
    def __init__(self):
        super().__init__()

    @staticmethod
    def _load_segmap(segmentation_map: Path) -> Dataset:
        """Read the segmentation mapping from the dcmqii standard json

        Args:
            segmentation_map (Path): Path to the json file

        Returns:
            Dataset: dataset information extracted from the json
        """
        assert os.path.exists(segmentation_map), "Seg mapping `.json` missing"

        assert verify_label_dcmqii_json(
            segmentation_map
        ), "Seg mapping `.json` not confirming to DCIM-QII standard, "

        return pydicom_seg.template.from_dcmqi_metainfo(segmentation_map)

    def _check_all_dicoms(self, dcmfiles: List[Path], seg: np.ndarray) -> List[Path]:
        """Verifies if the number of dicoms and the layers in segmentation match. Also sorts
            DICOMs based on the instance number.

        Args:
            dcmfiles (List[Path]): list of path to original dicom files
            seg (np.ndarray): 3d numpy array of a segmentation

        Returns:
            List[Path]: sorted list of dicoms based on the order
        """
        assert len(dcmfiles) in list(
            seg.shape
        ), f"""Need 1 DICOM per slice of NifTi;
        Found {len(dcmfiles)} DICOMS for {len(seg)} NifTi slice"""

        z_locs = [pydicom.dcmread(path).InstanceNumber for path in dcmfiles]

        return self.sort_order(z_locs, dcmfiles)

    @staticmethod
    def _check_all_lables(seg_map: Dataset, segImage: np.ndarray):
        """Check the integrity of the labels in the segmentation and the mapping provided in the json

        Args:
            seg_map (Dataset): mapping extracted from the corresponding json
            segImage (np.ndarray): 3d nifti segmentation

        Raises:
            ValueError: the segmentation is not found in the json
        """
        total_unique_segs = [
            seg for seg in list(np.unique(np.uint8(segImage))) if seg != 0
        ]

        assert len(seg_map.SegmentSequence) >= len(
            total_unique_segs
        ), "Not all the segmentation have a mapping in the json"
        # check all individual labels exist
        max_length_seq = len(seg_map.SegmentSequence)
        all_seq = [
            seg_map.SegmentSequence[i].SegmentNumber for i in range(max_length_seq)
        ]
        for seg in total_unique_segs:
            found = seg in all_seq
            if not found:
                raise ValueError(
                    f"No Segmentation mapping found for label {seg} in json"
                )

    @staticmethod
    def _create_dicomseg(
        seg_map: Dataset,
        segImage: np.ndarray,
        dcmImage: Union[FileDataset, List[FileDataset]],
    ) -> SegmentationDataset:
        """create a dicomseg for storage

        Args:
            seg_map (Dataset): Dataset info extraced from the mapping json
            segImage (np.ndarray): a single slice of the segmentation nifti image
            dcmImage (Union[FileDataset,List[FileDataset]]): input dicom to which
             the dicomseg is to be linked to, if a list a multilayer dicomseg will
             be created, else a single dicomseg for a single layer will be created

        Returns:
            SegmentationDataset: created dicomseg file
        """

        # convert to itk image
        segImage_itk = sitk.GetImageFromArray(segImage.astype(np.uint8))

        writer = pydicom_seg.MultiClassWriter(
            template=seg_map,
            inplane_cropping=False,  # Crop image slices to the minimum bounding box on
            # x and y axes. Maybe not supported by other frameworks.
            skip_empty_slices=True,  # Don't encode slices with only zeros
            skip_missing_segment=True,  # If a segment definition is missing in the
            # template, then raise an error instead of
            # skipping it.
        )

        # add fake storage info if necessary
        appendedImagePosition = False
        if type(dcmImage) == FileDataset:
            try:
                dcmImage.ImagePositionPatient
            except Exception:
                appendedImagePosition = True
                dcmImage.ImagePositionPatient = [0, 0, 0]
            # write the image
            dcmseg = writer.write(segImage_itk, source_images=[dcmImage])

            # correct the acquition time and other info if neccesary
            dcmseg.AcquisitionTime = dcmImage.AcquisitionTime
        elif type(dcmImage) == list:
            dcmseg = writer.write(segImage_itk, source_images=dcmImage)
            # correct the acquition time and other info if neccesary
            dcmseg.AcquisitionTime = dcmImage[0].AcquisitionTime

        if appendedImagePosition:
            dcmseg.ImagePositionPatient = None

        return dcmseg

    def _store_singlelayer_dicomseg(
        self,
        sorted_dcmfiles: List[Path],
        seg_map: Dataset,
        seg: np.ndarray,
        out_folder: Path,
    ) -> List[Path]:
        """stores each individual layer as a single dcm

        Args:
            sorted_dcmfiles (List[Path]): list of paths of all the source
             dicom files sorted by z-axis
            seg_map (Dataset): Dataset info extraced from the mapping json
            seg (np.ndarray): numpy array from a  segmentation nifti image
            out_folder (Path): path to folder to store the output to

        Returns:
            List[Path]: path to each individual dcm
        """
        # list to contain all the output paths of the dicomseg created
        out_list = []

        for i in range(len(sorted_dcmfiles)):

            # for non-empty segmenetation only
            non_zero_labels = np.unique(seg[..., i : i + 1]) != 0  # noqa
            if sum(non_zero_labels) > 0:
                dcm_file = Path(sorted_dcmfiles[i])
                dcm = pydicom.read_file(dcm_file)
                dcmseg = self._create_dicomseg(
                    seg_map, seg[..., i : i + 1], dcm  # noqa
                )
                out_dcmfile = Path(os.path.join(out_folder, dcm_file.name))
                dcmseg.save_as(out_dcmfile)
                out_list.append(out_dcmfile)

        return out_list

    def _store_multilayer_dicomseg(
        self,
        sorted_dcmfiles: List[Path],
        seg_map: Dataset,
        seg: np.ndarray,
        out_folder: Path,
    ) -> List[Path]:
        """stores all individual layer as a single multilayer dcm

        Args:
            sorted_dcmfiles (List[Path]): list of paths of all the source
             dicom files sorted by z-axis
            seg_map (Dataset): Dataset info extraced from the mapping json
            seg (np.ndarray): numpy array from a  segmentation nifti image
            out_folder (Path): path to folder to store the output to

        Returns:
            List[Path]: path to dcmseg
        """
        sorted_dcm = [pydicom.read_file(dcm_file) for dcm_file in sorted_dcmfiles]
        dcmseg = self._create_dicomseg(seg_map, seg, sorted_dcm)
        out_dcmfile = Path(os.path.join(out_folder, Path(sorted_dcmfiles[0]).name))
        dcmseg.save_as(out_dcmfile)

        return [out_dcmfile]

    def multilabel_converter(
        self, segfiles=List[Path], dcmfiles=List[Path]
    ) -> List[Path]:
        # all_individual_segs = [nib.load(file).get_fdata() for file in segfiles]
        # de-priorisited for now

        raise NotImplementedError(
            "Multilabel NifTi s to DICOMSeg cannot to be handled yet!!"
        )

    def multiclass_converter(
        self,
        segfile: Path,
        segMapping: Path,
        dcmfiles: List[Path],
        multiLayer: bool = False,
    ) -> List[Path]:
        """Convert a given nifti segmentation to dicomseg for multiclass segmentations

        Args:
            segfile (Path): path to the nifti segmentation file
            segMapping (Path): path to the dcmqii format segmentation mapping json
            dcmfiles (List[Path]): list of paths of all the source dicom files
            multiLayer (bool, optional): create a single multilayer dicomseg. Defaults to False.

        Returns:
            List[Path]: list of paths of all generated dicomseg files
        """

        # load the segmentation mapping
        seg_map = self._load_segmap(segMapping)
        # load the segmentation and verify if all dicoms exist
        seg = nib.load(segfile).get_fdata()
        sorted_dcmfiles = self._check_all_dicoms(dcmfiles, seg)

        self._check_all_lables(seg_map, seg)

        # create folder to store the dicomsegs
        parent_dir = Path(segfile).parent
        out_folder = Path(os.path.join(parent_dir, "dicomseg"))
        os.makedirs(out_folder, exist_ok=True)

        # create store individual dicomseg
        if multiLayer:
            out_list = self._store_multilayer_dicomseg(
                sorted_dcmfiles, seg_map, seg, out_folder
            )
        else:
            out_list = self._store_singlelayer_dicomseg(
                sorted_dcmfiles, seg_map, seg, out_folder
            )

        return out_list



# Defining convenience import shortcuts for those classes which are
# supposed to be used publicly
from .measurementReport import MeasurementReport
from .measurementGroup import MeasurementGroup
from .measurementItem import VolumeMeasurementItem, MeanADCMeasurementItem
from .codeSequences import CodeSequence, Finding, FindingSite, ProcedureReported




# helper script to populate Slicer integer labels in the
#   segmentation context JSON file from Slicer LUT CSV file
#
# Usage:
# $ python resources/add_slicer_ids.py resources/SlicerGenericAnatomy.csv \
#   SegmentationCategoryTypeModifier-SlicerGeneralAnatomy.json \
#   SegmentationCategoryTypeModifier-SlicerGeneralAnatomy_updated.json

import csv, sys, pandas, json, copy

def lookup_slicer_int_from_text(slicer_lut, slicer_text_label):
  try:
    slicer_int_label = int(slicer_lut[slicer_lut["Text Label"] == slicer_text_label]["Integer Label"].values[0])
  except IndexError:
    print("Failed to find "+slicer_text_label+" in the CSV file")
    raise IndexError

  return slicer_int_label

slicer_lut = pandas.read_csv(sys.argv[1])
slicer_lut = slicer_lut.apply(lambda x: x.str.strip() if x.dtype == "object" else x)

with open(sys.argv[2], 'r') as json_file:
  seg_context = json.load(json_file)

new_seg_context = copy.deepcopy(seg_context)

for category_id,category in enumerate(seg_context["SegmentationCodes"]["Category"]):
  for type_id,type in enumerate(category["Type"]):
    try:
      slicer_label = type["3dSlicerLabel"]
    except KeyError:
      if not "Modifier" in type.keys(): # otherwise it's not a concern
        print("Failed to find 3dSlicerLabel in "+str(type))

    slicer_int_label = lookup_slicer_int_from_text(slicer_lut, slicer_label)
    new_seg_context["SegmentationCodes"]["Category"][category_id]["Type"][type_id]["3dSlicerIntegerLabel"] = slicer_int_label

    if "Modifier" in type.keys():
      for modifier_id,modifier in enumerate(type["Modifier"]):
        try:
          slicer_label = modifier["3dSlicerLabel"]
        except KeyError:
          print("Failed to find 3dSlicerLabel in modifier for "+json.dumps(type, indent=2))
        slicer_int_label = lookup_slicer_int_from_text(slicer_lut, slicer_label)
        new_seg_context["SegmentationCodes"]["Category"][category_id]["Type"][type_id]["Modifier"][modifier_id]["3dSlicerIntegerLabel"] = slicer_int_label

with open(sys.argv[3],"w") as new_seg_context_file:
  json.dump(new_seg_context, new_seg_context_file, indent=2)




class CodeSequence(object):

  def __init__(self, codeMeaning, codingSchemeDesignator, codeValue):
    self.CodeValue = codeValue
    self.CodingSchemeDesignator = codingSchemeDesignator
    self.CodeMeaning = codeMeaning


class Finding(CodeSequence):

  def __init__(self, segmentedStructure):
    if segmentedStructure == "NormalROI_PZ_1":
      super().__init__("Normal", "SCT", "17621005")
    elif segmentedStructure == "PeripheralZone":
      super().__init__("Entire", "SCT", "255503000")
    elif segmentedStructure == "TumorROI_PZ_1":
      super().__init__("Abnormal", "SCT", "263654008")
    elif segmentedStructure == "WholeGland":
      super().__init__("Entire Gland", "SCT", "714645007")
    else:
      raise ValueError("Segmented Structure Type {} is not supported yet. Build your own Finding code sequence using the class CodeSequence".format(segmentedStructure))
  

class FindingSite(CodeSequence):
  
  def __init__(self, segmentedStructure):
    if segmentedStructure == "NormalROI_PZ_1":
      super().__init__("Peripheral zone of the prostate", "SCT", "279706003")
    elif segmentedStructure == "PeripheralZone":
      super().__init__("Peripheral zone of the prostate", "SCT", "279706003")
    elif segmentedStructure == "TumorROI_PZ_1":
      super().__init__("Peripheral zone of the prostate", "SCT", "279706003")
    elif segmentedStructure == "WholeGland":
      super().__init__("Prostate", "SCT", "41216001")
    else:
      raise ValueError("Segmented Structure Type {} is not supported yet. Build your own FindingSite code sequence using the class CodeSequence".format(segmentedStructure))


class ProcedureReported(CodeSequence):

  def __init__(self, codeMeaning):
    if codeMeaning == "Multiparametric MRI of prostate":
      super().__init__("Multiparametric MRI of prostate", "DCM", "126021")
    else:
      raise ValueError("Procedure Type {} is not supported yet. Build your own ProcedureReported code sequence using the class CodeSequence".format(codeMeaning))




import json
import jsonschema
import os


def _create_validator() -> jsonschema.Draft4Validator:
    """Create a JSON validator instance from dcmqi schema files.
    In order to allow offline usage, the required schemas a pre-loaded from the
    dcmqi repository located at `nekton/externals/dcmqi`.
    Returns:
        A `jsonschema.Draft4Validator` with a pre-loaded schema store.
    """
    # Load both common and segmentation schema files
    schemas_dir = os.path.join(
        os.path.dirname(os.path.dirname(__file__)),
        "externals/dcmqi/doc/schemas",
    )
    seg_schema_path = os.path.join(schemas_dir, "seg-schema.json")
    with open(seg_schema_path) as ifile:
        seg_schema = json.load(ifile)
    with open(os.path.join(schemas_dir, "common-schema.json")) as ifile:
        common_schema = json.load(ifile)

    # Create validator with pre-loaded schema store
    return jsonschema.Draft4Validator(
        seg_schema,
        resolver=jsonschema.RefResolver(
            base_uri="file://" + seg_schema_path,
            referrer=seg_schema,
            store={seg_schema["id"]: seg_schema, common_schema["id"]: common_schema},
        ),
    )


def write_json(dictionary: dict, path: str) -> str:
    """write a dictionary as json

    Args:
        dictionary (dict): a dictionary that has to be converted to json
        path (str): path to output json file

    Returns:
        str: path to output json file
    """
    try:
        json_object = json.dumps(dictionary, indent=4)
        with open(path, "w") as outfile:
            outfile.write(json_object)
    except Exception as err:
        raise RuntimeError(f"Unable to write json: {err}")

    return path


def read_json(path: str) -> dict:
    """read a json as a dictionary

    Args:
        path (str): path to a json file

    Returns:
        dict: the dictionary read from a json file
    """
    try:
        with open(path, "r") as openfile:
            json_object = json.load(openfile)
    except Exception as err:
        raise RuntimeError(f"Unable to read json: {err}")

    return json_object


def verify_label_dcmqii_json(path: str) -> bool:
    try:
        with open(path, "r") as openfile:
            data = json.load(openfile)
    except Exception:
        raise NameError(f"Invalid Json at {path}")

    validator = _create_validator()

    if not validator.is_valid(data):
        raise TypeError("This schema not supported! only dcmqi JSON schema supported")
    return True



import json, sys, csv, re

class Term:
  def __init__(self,s):
    #print s,' maps to ',
    try:
      mo = re.search("\((.*),(.*),(\".*\")\)",s)

      self.code = mo.group(1)
      self.codingScheme = mo.group(2)
      self.meaning = mo.group(3)
    except:
      self.code = ""
      self.codingScheme = ""
      self.meaning = ""
    #print self.getStr()
  def toStr(self):
    return '('+self.code+','+self.codingScheme+','+self.meaning+')'

class RGB:
  def __init__(self,s):
    try:
      mo = re.search("rgb\(([0-9]*),([0-9]*),([0-9]*)\)",s)
      self.rgb = [mo.group(1),mo.group(2),mo.group(3)]
    except:
      self.rgb = ""
  def toStr(self):
    return str(self.rgb)

def addColor(codeDict,jsonDict,c,t,m=-1):
  jsonCat = jsonCodes["SegmentationCodes"]["Category"][c]["codeValue"]
  jsonType = jsonCodes["SegmentationCodes"]["Category"][c]["Type"][t]["codeValue"]
  if m != -1:
    jsonMod = jsonCodes["SegmentationCodes"]["Category"][c]["Type"][t]["Modifier"][m]["codeValue"]
  else:
    jsonMod = ""
  key = jsonCat+","+jsonType+","+jsonMod

  if key in codeDict:
    rgb = codeDict[key][-2].rgb
    slicerName = codeDict[key][-3]
    codeDict[key][-1] = True # mark this entry as matched
    if m != -1:
      jsonCodes["SegmentationCodes"]["Category"][c]["Type"][t]["Modifier"][m]["recommendedDisplayRGBValue"] = rgb
      jsonCodes["SegmentationCodes"]["Category"][c]["Type"][t]["Modifier"][m]["3dSlicerLabel"] = slicerName
    else:
      jsonCodes["SegmentationCodes"]["Category"][c]["Type"][t]["recommendedDisplayRGBValue"] = rgb
      jsonCodes["SegmentationCodes"]["Category"][c]["Type"][t]["3dSlicerLabel"] = slicerName
    return True
  else:
    return False

def copyNonListItems(src,dest,destKey=None):
  if destKey:
    dest[destKey] = {}
  for k in src.keys():
    if type(k) != 'list':
      if not destKey:
        dest[k] = src[k]
      else:
        dest[destKey][k] = src[k]

if len(sys.argv)<3:
  print "Usage: ",sys.argv[0]," SegmentationCodesJSON SegmentationCodesCSV <outputJSON>"
  print "  where SegmentationCodesJSON is the SegmentationCategoryType JSON"
  print "  file from doc/segContext, and SegmentationCodesCSV is the CSV file" print "  imported from the first sheet here https://goo.gl/vuANnw"

inputJSONFileName = sys.argv[1]
inputCSVFileName = sys.argv[2]

if len(sys.argv)>3:
  outputJSONFileName = sys.argv[3]
else:
  outputJSONFileName = None

jsonCodes = json.loads(open(inputJSONFileName).read())
#jsonCodes = jsonCodes["SegmentationCodes"]

codeDict = {}

with open(inputCSVFileName) as csvfile:
  csvreader = csv.reader(csvfile)
  for row in csvreader:
    slicerName = row[1].strip()
    segCategory = Term(row[6])
    segType = Term(row[7])
    segModifier = Term(row[9])
    segRGB = RGB(row[10])
    #if segCategory != None and segCategory != ' ':
    #  print '*'+segCategory+'*',segType,segModifier,segRGB
    key = segCategory.code+","+segType.code+","+segModifier.code
    if segCategory.code != "":
      # last item indicates this entry has not been matched to the JSON dataset
      codeDict[key] = [segCategory,segType,segModifier,slicerName,segRGB,False]

print "Total valid entries in CSV:",len(codeDict.keys())

totalJSONEntries = 0
matchedCodes = []
for c in range(len(jsonCodes["SegmentationCodes"]["Category"])):
  for t in range(len(jsonCodes["SegmentationCodes"]["Category"][c]["Type"])):
    if "Modifier" in jsonCodes["SegmentationCodes"]["Category"][c]["Type"][t]:
      for m in range(len(jsonCodes["SegmentationCodes"]["Category"][c]["Type"][t]["Modifier"])):
        matched = addColor(codeDict,jsonCodes,c,t,m)
        totalJSONEntries = totalJSONEntries+1
        if matched:
          matchedJsonCodes.append([c,t,m])
    else:
      matched = addColor(codeDict,jsonCodes,c,t)
      totalJSONEntries = totalJSONEntries+1
      if matched:
        matchedJsonCodes.append([c,t])

# copy matched codes to a new JSON
matchedCodesJson = {}
matchedCodesJson["SegmentationCodes"] = {}
matchedCodesJson["SegmentationCodes"]["Category"] = {}
# look up if a category with the given key/coding scheme already exists
categoryLookupByKey = {}

if len(matchedCodes):
  for t in matchedJsonCodes:
    cCat = jsonCodes["SegmentationCodes"]["Category"][t[0]]]
    cType = jsonCodes["SegmentationCodes"]["Category"][t[0]]]["Type"][t[1]]
    if len(t)==3:
      cMod = jsonCodes["SegmentationCodes"]["Category"][t[0]]]["Type"][t[1]]["Modifier"][t[2]]
    else:
      cMod = None



print "Total JSON entries:",totalJSONEntries

unmatchedEntries = [l for l in codeDict.values() if l[-1]==False]
print "Total unmatched entries:",len(unmatchedEntries)

for e in unmatchedEntries:
  print e[0].toStr(),e[1].toStr(),e[2].toStr()

if outputJSONFileName:
  f = open(outputJSONFileName,"w")
  f.write(json.dumps(jsonCodes,indent=2))



import json, sys
import jsoncompare
import ast


if len(sys.argv) < 3:
  sys.exit('Error')
json1 = json.loads(open(sys.argv[1],'r').read())
json2 = json.loads(open(sys.argv[2],'r').read())

ignoredKeys = ["@schema"]

try:
    ignoredKeys += ast.literal_eval(sys.argv[3])
except IndexError:
    pass

json2 = json.loads(open(sys.argv[2],'r').read())

(check,stack) = jsoncompare.are_same(json1,json2,
                                     ignore_list_order_recursively=False,
                                     ignore_missing_keys=False,
                                     ignore_value_of_keys=ignoredKeys)

if not check:
  print(stack)
  sys.exit(1)



from pydicom.sr import _snomed_dict
import os
import re
import pydicom.sr._snomed_dict

folder = "E:\\work\\QIICR\\dcmqi"
Out_Folder = "E:\\work\\QIICR\\renamed_dcmqi"


def recursive_file_find(address, regexp):
    filelist = os.listdir(address)
    approvedlist=[]
    for filename in filelist:
        fullpath = os.path.join(address, filename)
        if os.path.isdir(fullpath):
            approvedlist.extend(recursive_file_find(fullpath,  regexp))
        elif re.match(regexp, fullpath) is not None:
            approvedlist.append(fullpath)
    return approvedlist


def GetFileString(filename):
    File_object = open(filename, "r")
    try:
        Content = File_object.read()
    except:
        print("Couldn't read the file")
        Content = ""
    File_object.close()
    return Content


def WriteTextInFile(filename, txt):
    folder = os.path.dirname(filename)
    if not os.path.exists(folder):
        os.makedirs(folder)
    File_object = open(filename, "w")
    File_object.write(txt)
    File_object.close()


def FindRegex(regexp, text, extend=[0, 0], printout=False):
    found_iters = re.finditer(regexp, text)
    founds = list(found_iters)
    ii = []
    for mmatch in founds:
        yy = text[mmatch.start() - extend[0]:mmatch.end() + extend[1]]
        counter = "[%04d ]" % len(ii)
        if (printout):
            print(counter + yy)
        ii.append(yy)
    return ii


def ReplaceQuotedText(find_text, rep_text, text):
    pattern = "(\"\s*" + find_text + "\s*\")|('\s*" + find_text + "\s*')"
    replacement = "\"" + rep_text + "\"";
    new_text = re.sub(pattern, replacement, text)
    View = ShowReplaceMent(pattern, replacement, text)
    return [new_text, View]


def FindAndReplace(find_text, rep_text, text):
    newtext = re.sub(find_text, rep_text, text)
    x = ShowReplaceMent(find_text, rep_text, text)
    return [newtext, x]


def ShowReplaceMent(find_text, rep_text, text):
    output = []
    text_seq = FindRegex("\\n.*(" + find_text + ").*\\n", text, [-1, -1])
    for line_txt in text_seq:
        found_iters = re.finditer(find_text, line_txt)
        founds = list(found_iters)
        if len(founds) > 0:
            mmatch = founds[0]
            yy = line_txt[:mmatch.start()] + \
                 "{ [" + line_txt[mmatch.start():mmatch.end()] + "]-->[" + rep_text + "] }" + \
                 line_txt[mmatch.end():]
            output.append(yy)
    return output


dict = _snomed_dict.mapping["SCT"]
details = []

# recursive_file_find(folder, all_files, "(.*\\.cpp$)|(.*\\.h$)|(.*\\.json$)")
all_files = recursive_file_find(folder, "(?!.*\.git.*)")
for f, jj in zip(all_files, range(1, len(all_files))):
    f_content = (GetFileString(f))
    if len(f_content) == 0:
        continue

    [f_content, x] = ReplaceQuotedText("SCT", "SCT", f_content)
    details = x
    [f_content, x] = FindAndReplace(",\s*SRT\s*,", ",SCT,", f_content)
    details.extend(x)
    [f_content, x] = FindAndReplace("SCT", " SCT ", f_content)
    details.extend(x)
    [f_content, x] = FindAndReplace("_SCT_", "_SCT_", f_content)
    details.extend(x)
    [f_content, x] = FindAndReplace("sct.h", "sct.h", f_content)
    details.extend(x)

    for srt_code, sct_code in dict.items():
        # f_content = ReplaceQuotedText(srt_code, sct_code, f_content)
        [f_content, x] = FindAndReplace(srt_code, sct_code, f_content)
        details.extend(x)
    if len(details) == 0:
        continue
    edited_file_name = f.replace(folder, Out_Folder)
    edited_file_log = f.replace(folder, os.path.join(Out_Folder, '..\\log')) + ".txt"
    WriteTextInFile(edited_file_name, f_content)
    print("------------------------------------------------------------------------")
    f_number = "(file %03d ) " % jj
    print(f_number + f)
    logg = ""
    for m, c in zip(details, range(0, len(details))):
        indent = "\t\t\t%04d" % c
        logg += (indent + m + "\n")
    if len(logg) != 0:
        WriteTextInFile(edited_file_log, logg)

print("the find/replace process finished ...")



import pydicom, sys
from colorama import Fore, Style, init

# colorama
init()

d = pydicom.read_file(sys.argv[1])
if len(sys.argv)>2:
  frame = int(sys.argv[2])-1
  print("Dumping frame "+str(frame))
else:
  frame = None
print(d.Rows)
print(d.Columns)
print(d.NumberOfFrames)

totalPixels = int(d.Rows*d.Columns*d.NumberOfFrames/8)
if totalPixels%8:
  totalPixels = totalPixels + 1
totalPixels = totalPixels + (totalPixels % 2)
print("Total pixels expected: %i" % totalPixels)

print("Total pixels actual: %i" % len(d.PixelData))

if not frame is None:
  frames = [frame]
else:
  frames = range(d.NumberOfFrames)



import numpy as np
unpacked = np.unpackbits(np.frombuffer(d.PixelData,dtype=np.uint8))

print("With numpy unpackbits:")

for f in frames:
  print("Frame %i" % f)
  for i in range(d.Rows):
    for j in range(d.Columns):
      pixelNumber = f*d.Rows*d.Columns+i*d.Columns+j
      if int(pixelNumber/8)%2:
        sys.stdout.write(Fore.RED)
      else:
        sys.stdout.write(Fore.WHITE)
      if unpacked[pixelNumber]:
        sys.stdout.write("X")
      else:
        sys.stdout.write(".")
    print("")

print("\nWith manual unpacking:")
for f in frames:
  print("Frame %i" % f)
  for i in range(d.Rows):
    for j in range(d.Columns):
      pixelNumber = f*d.Rows*d.Columns+i*d.Columns+j
      byteNumber = int(pixelNumber/8)
      bitPosition = pixelNumber % 8
      if byteNumber%2:
        sys.stdout.write(Fore.RED)
      else:
        sys.stdout.write(Fore.WHITE)
      if (d.PixelData[byteNumber] >> bitPosition) & 1:
        sys.stdout.write("X")
      else:
         sys.stdout.write(".")
    print("")
print(Style.RESET_ALL)



import subprocess
import os

from os.path import abspath
from os.path import dirname as d

parent_dir = f"{d(d(abspath(__file__)))}"
PATH_TO_BIN = os.path.join(parent_dir, "bins/dcm2nii")


def make_exec_bin():
    """makes the binary executable on host PC"""
    process = subprocess.Popen(
        ["chmod", "+x", PATH_TO_BIN],
        stdout=subprocess.PIPE,
        universal_newlines=True,
    )
    process.communicate()


def run_bin(path: str, outpath:str = None):
    """run the binary on a given directory

    Args:
        path ([str]): directory where dicom exists
    """
    if outpath is None:
        process = subprocess.Popen(
            [PATH_TO_BIN, "-z", "y", path],
            stdout=subprocess.PIPE,
            universal_newlines=True,
        )
    else:
        process = subprocess.Popen(
            [PATH_TO_BIN, "-z", "y","-o", outpath, path],
            stdout=subprocess.PIPE,
            universal_newlines=True,
        )
    process.communicate()



import pydicom
from pydicom.errors import InvalidDicomError


def is_file_a_dicom(file: str) -> bool:
    """function to check if a given file is a DICOM or not

    Args:
        file (str): path to a file

    Returns:
        bool: returns True if a file is a DICOM else False
    """

    try:
        pydicom.read_file(file)
    except InvalidDicomError:
        return False
    return True



from .utils.json_helpers import write_json
import os
from pathlib import Path
from typing import List


class BaseConverter:
    def __init__(self):
        pass

    @staticmethod
    def write_dict_json(directory: Path, data_dict: dict, name: str) -> Path:
        """write a dictionary type to json file

        Args:
            directory (Path): directory where the json has to be stored
            data_dict (dict): dictionary to be made json
            name (Path): name of the json file

        Returns:
            Path: the path of the stored json
        """
        if ".json" not in name:
            name += ".json"
        output_filepath = Path(os.path.join(directory, name))
        write_json(data_dict, output_filepath)
        return output_filepath

    @staticmethod
    def sort_order(zloc_paths: list, nii_paths: List[Path]) -> List[Path]:
        # Stack the portions in correct order
        try:
            sorted_nii_list = [x for _, x in sorted(zip(zloc_paths, nii_paths))]
        except Exception:
            raise NameError("Folder has Multiple Series; Cannot handle it yet!!")

        return sorted_nii_list



import json

from .measurementGroup import MeasurementGroup
from .measurementItem import MeasurementItem
from .codeSequences import CodeSequence

class MeasurementReport(object):
  """
  Data structure plus convenience methods to create measurment reports following
  the required format to be processed by the DCMQI tid1500writer tool (using the
  JSON export of this).
  """

  def __init__(self, seriesNumber, compositeContext, dicomSourceFileList, timePoint, 
               seriesDescription = "Measurements", procedureReported = None):
    self.SeriesDescription = str(seriesDescription)
    self.SeriesNumber = str(seriesNumber)
    self.InstanceNumber = "1"

    self.compositeContext = [compositeContext]

    self.imageLibrary = dicomSourceFileList

    self.observerContext = {
      "ObserverType": "PERSON",
      "PersonObserverName": "Reader01"
    }

    if procedureReported:
      self.procedureReported = procedureReported

    self.VerificationFlag = "VERIFIED"
    self.CompletionFlag = "COMPLETE"

    self.activitySession = "1"
    self.timePoint = str(timePoint)

    self.Measurements = []


  def addMeasurementGroup(self, measurementGroup):
    self.Measurements.append(measurementGroup)

  
  def exportToJson(self, fileName):
    with open(fileName, 'w') as fp:
      json.dump(self._getAsDict(), fp, indent = 2)


  def getJsonStr(self):
    return json.dumps(self._getAsDict(), indent = 2)


  def _getAsDict(self):
    # This is a bit of a hack to get the "@schema" in there, didn't figure out how to 
    # do this otherwise with json.dumps. If this wasn't needed I could just dump
    # the json directly with my custom encoder.
    jsonStr = json.dumps(self, indent = 2, cls = self._MyJSONEncoder)
    tempDict = json.loads(jsonStr)
    outDict = {}
    outDict["@schema"] = "https://raw.githubusercontent.com/qiicr/dcmqi/master/doc/schemas/sr-tid1500-schema.json#"
    outDict.update(tempDict)
    return outDict

  # Inner private class to define a custom JSON encoder for serializing MeasurmentReport
  class _MyJSONEncoder(json.JSONEncoder):
    def default(self, obj):
      if (isinstance(obj, MeasurementReport) or 
          isinstance(obj, MeasurementGroup) or
          isinstance(obj, MeasurementItem) or
          isinstance(obj, CodeSequence)):
        return obj.__dict__
      else:
        return super(MyEncoder, self).default(obj)



class MeasurementGroup(object):
  """
  Data structure plus convenience methods to create measurment groups following
  the required format to be processed by the DCMQI tid1500writer tool. Use this
  to populate the Measurements list in :class:`MeasurementReport`.
  """

  def __init__(self, 
               trackingIdentifier, trackingUniqueIdentifier, referencedSegment, 
               sourceSeriesInstanceUID, segmentationSOPInstanceUID,
               finding, findingSite):
    self.TrackingIdentifier = trackingIdentifier
    self.TrackingUniqueIdentifier = trackingUniqueIdentifier
    self.ReferencedSegment = referencedSegment
    self.SourceSeriesForImageSegmentation = sourceSeriesInstanceUID
    self.segmentationSOPInstanceUID = segmentationSOPInstanceUID
    self.Finding = finding
    self.FindingSite = findingSite
    self.measurementItems = []

  def addMeasurementItem(self, measurementItem):
    self.measurementItems.append(measurementItem)

    





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nekton',
 'nekton.externals.dcmqi.doc.segContexts.resources',
 'nekton.externals.dcmqi.util',
 'nekton.externals.dcmqi.util.junit',
 'nekton.externals.dcmqi.util.measurementReportUtils',
 'nekton.utils']

package_data = \
{'': ['*'],
 'nekton': ['bins/*',
            'externals/dcmqi/*',
            'externals/dcmqi/.circleci/*',
            'externals/dcmqi/CMake/*',
            'externals/dcmqi/CMakeExternals/*',
            'externals/dcmqi/Design/*',
            'externals/dcmqi/apps/*',
            'externals/dcmqi/apps/paramaps/*',
            'externals/dcmqi/apps/paramaps/Testing/*',
            'externals/dcmqi/apps/seg/*',
            'externals/dcmqi/apps/seg/Testing/*',
            'externals/dcmqi/apps/sr/*',
            'externals/dcmqi/apps/sr/Testing/*',
            'externals/dcmqi/data/paramaps/*',
            'externals/dcmqi/data/segmentations/*',
            'externals/dcmqi/data/segmentations/23x38x3/image/*',
            'externals/dcmqi/data/segmentations/23x38x3/multiframe/*',
            'externals/dcmqi/data/segmentations/23x38x3/nrrd/*',
            'externals/dcmqi/data/segmentations/23x38x3/screenshots/*',
            'externals/dcmqi/data/segmentations/23x38x3/seg/*',
            'externals/dcmqi/data/segmentations/24x38x3/image/*',
            'externals/dcmqi/data/segmentations/24x38x3/multiframe/*',
            'externals/dcmqi/data/segmentations/24x38x3/nrrd/*',
            'externals/dcmqi/data/segmentations/24x38x3/screenshots/*',
            'externals/dcmqi/data/segmentations/24x38x3/seg/*',
            'externals/dcmqi/data/segmentations/ct-3slice/*',
            'externals/dcmqi/data/sr-example/*',
            'externals/dcmqi/doc/*',
            'externals/dcmqi/doc/definitions/*',
            'externals/dcmqi/doc/examples/*',
            'externals/dcmqi/doc/pmContexts/*',
            'externals/dcmqi/doc/schemas/*',
            'externals/dcmqi/doc/segContexts/*',
            'externals/dcmqi/docker/*',
            'externals/dcmqi/docker/dcmqi/*',
            'externals/dcmqi/docker/dcmqi/imagefiles/*',
            'externals/dcmqi/include/dcmqi/*',
            'externals/dcmqi/jsoncpp/*',
            'externals/dcmqi/jsoncpp/json/*',
            'externals/dcmqi/libsrc/*']}

install_requires = \
['SimpleITK==2.2.0',
 'nibabel>=3.2.2,<4.0.0',
 'numpy==1.23.3',
 'pydicom-seg==0.3.0',
 'pydicom>=2.3.0,<3.0.0']

setup_kwargs = {
    'name': 'nekton',
    'version': '0.2.6rc2',
    'description': 'A python package for DICOM to NifTi and NifTi to DICOM-SEG and GSPS conversion',
    'long_description': '# Nekton\n[![Python Application Testing](https://github.com/deepc-health/nekton/actions/workflows/tests.yml/badge.svg?branch=master)](https://github.com/deepc-health/nekton/actions/workflows/tests.yml)[![Test and Release](https://github.com/deepc-health/nekton/actions/workflows/release.yml/badge.svg?branch=master)](https://github.com/deepc-health/nekton/actions/workflows/release.yml)\n[![Python Versions](https://img.shields.io/pypi/pyversions/nekton.svg)](https://pypi.org/project/nekton/)[![Package version](https://img.shields.io/pypi/v/nekton?color=%2334D058&label=pypi%20package)](https://pypi.org/project/nekton/)\n\n> A python package for DICOM to NifTi and NifTi to DICOM-SEG and GSPS conversion\n\n## SETUP\n\nThe python package is available for use on PyPI. It can be setup simply via pip\n\n```bash\npip install nekton\n```\n\nTo the check the setup, simply check the version number of the `nekton` package by\n\n```bash\npython -c \'import nekton; print(nekton.__version__)\'\n```\n\n## DICOM to NifTi\n\nThe DICOM to NifTi conversion in the package is based on a wrapper around the [dcm2niix](https://github.com/rordenlab/dcm2niix) software.\n\n### Usage\n\n```python\nfrom nekton.dcm2nii import Dcm2Nii\nconverter = Dcm2Nii()\nconverted_files = converter.run(dicom_directory=\'/test_files/CT5N\',  out_directory=\'/test_files/CT5N\', name=\'Test\')\n# Converted 5 DCM to Nifti; Output stored @ /test_files/CT5N\nprint(converted_files)\n# [\'/test_files/CT5N/Test_SmartScore_-_Gated_0.5_sec_20010101000000_5.nii.gz\']\n```\n\nParameters `converter.run`:\n\n- `dicom_directory (Path)`: path to directory with Dicoms\n- `dicom_directory (Path, optional)`: directory to store the output nifti\n- `name (str, optional)`: Name to be given to the output file. Defaults to "".\n\nReturns:\n\n- `List[Path]`: output list of Nifti files\n\n\n### Notes\n\n- The renaming functionality retains the [suffixes](https://github.com/rordenlab/dcm2niix/blob/master/FILENAMING.md) from the original program.\n- The BIDS sidecar json is retained as well.\n\n## NifTi to DICOM-SEG\n\nThe NifTi to DICOM-SEG within nekton converts incoming segmentation NifTi to DICOM-SEG. The matching of the segmentation index to a text label is \ndone via json file using the schema suggested by `dcmqi`. The json can be generated using the [gui](http://qiicr.org/dcmqi/#/seg) also an example can be seen [here](https://github.com/deepc-health/nekton/blob/master/tests/test_data/sample_segmentation/mapping.json). \n\nCurrently, `nekton` supports creation of multiclass DICOM-SEG of two types-\n\n- single layer DICOM-SEG, where each non-empty slice has an individual file\n- multi layer DICOM-SEG, where all the n slices are rolled into a single file\n\n### Usage\n\n1. NifTi to single layer DICOM-SEG\n\n```python\nfrom nekton.nii2dcm import Nii2DcmSeg\nimport glob\nconverter = Nii2DcmSeg()\npath_dcms = [path for path in glob.glob(dir_dcms)]\npath_mapping = "mapping.json"\npath_seg_nifti = "CT5N_segmentation.nii.gz"\ndcmsegs = converter_dcmseg.multiclass_converter(\n        segfile = path_seg_nifti, segMapping= path_mapping, dcmfiles =path_dcms, multiLayer=False\n    )\nprint (len(dcmsegs))\n# 3\n```\n\n2. NifTi to multi layer DICOM-SEG\n\n```python\nfrom nekton.nii2dcm import Nii2DcmSeg\nimport glob\nconverter = Nii2DcmSeg()\npath_dcms = [path for path in glob.glob(dir_dcms)]\npath_mapping = "mapping.json"\npath_seg_nifti = "CT5N_segmentation.nii.gz"\ndcmsegs = converter.multiclass_converter(\n        segfile = path_seg_nifti, segMapping= path_mapping, dcmfiles =path_dcms, multiLayer=True\n    )\nprint (len(dcmsegs))\n# 1\n```\n\nParameters `converter.multiclass_converter`:\n\n- `segfile (Path)`: path to the nifti segmentation file\n- `segMapping (Path)`: path to the dcmqii format segmentation mapping json\n- `dcmfiles (List[Path])`: list of paths of all the source dicom files\n- `multiLayer (bool, optional)`: create a single multilayer dicomseg. Defaults to False.\n\nReturns:\n\n- `List[Path]`: list of paths of all generated dicomseg files\n\n### Notes\n\n- Multilabel NifTi(in the form of a NifTi file for a single label) to DICOM-SEG is under development.\n\n## NifTi to GSPS\n\n```\nThis feature will be available in a future release of the nekton\n```\n',
    'author': 'a-parida12',
    'author_email': 'abhijeet@deepc.ai',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/deepc-health/nekton',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from lxml import etree
import StringIO
import sys

TAGfile = open(sys.argv[1]+"/Testing/TAG", 'r')
dirname = TAGfile.readline().strip()

xmlfile = open(sys.argv[1]+"/Testing/"+dirname+"/Test.xml", 'r')
xslfile = open(sys.argv[2], 'r')

xmlcontent = xmlfile.read()
xslcontent = xslfile.read()

xmldoc = etree.parse(StringIO.StringIO(xmlcontent))
xslt_root = etree.XML(xslcontent)
transform = etree.XSLT(xslt_root)

result_tree = transform(xmldoc)
print(result_tree)




class MeasurementItem(object):

  def __init__(self, value):
    self.value = self.convertNumericToDcmtkFittingString(value)

  def convertNumericToDcmtkFittingString(self, value):
    if isinstance(value, float) or isinstance(value, int):
      s = str(value)
      if (len(s) <= 16):
        return s
      elif (s.find(".") >= 0) and (s.find(".") < 15):
        return s[:16]
      else:
        raise ValueError("Value cannot be converted to 16 digits without loosing too much precision!")
    else:
      raise TypeError("Value to convert is not of type float or int")


class VolumeMeasurementItem(MeasurementItem):

  def __init__(self, value):
    super().__init__(value)
    self.quantity = {
      "CodeValue": "118565006", 
      "CodingSchemeDesignator": "SCT", 
      "CodeMeaning": "Volume"
    }
    self.units = {
      "CodeValue": "cm3",
      "CodingSchemeDesignator": "UCUM",
      "CodeMeaning": "cubic centimeter"
    }


class MeanADCMeasurementItem(MeasurementItem):
  def __init__(self, value):
    super().__init__(value)
    self.quantity = {    
      "CodeValue": "113041",
      "CodingSchemeDesignator": "DCM",
      "CodeMeaning": "Apparent Diffusion Coefficient"
    }
    self.units = {
      "CodeValue": "um2/s",
      "CodingSchemeDesignator": "UCUM",
      "CodeMeaning": "um2/s"
    }
    self.derivationModifier = {
      "CodeValue": "373098007",
      "CodingSchemeDesignator": "SCT",
      "CodeMeaning": "Mean"
    }



import json


class Stack:
    def __init__(self):
        self.stack_items = []

    def append(self, stack_item):
        self.stack_items.append(stack_item)
        return self

    def __repr__(self):
        stack_dump = ''
        for item in self.stack_items:
            stack_dump += str(item)
        return stack_dump

    def __str__(self):
        stack_dump = ''
        for item in self.stack_items:
            stack_dump += str(item)
        return stack_dump


class StackItem:
    def __init__(self, reason, expected, actual):
        self.reason = reason
        self.expected = expected
        self.actual = actual

    def __repr__(self):
        return 'Reason: {0}\nExpected:\n{1}\nActual:\n{2}' \
               .format(self.reason, _format_value(self.expected), _format_value(self.actual))

    def __str__(self):
        return '\n\nReason: {0}\nExpected:\n{1}\nActual:\n{2}' \
               .format(self.reason, _format_value(self.expected), _format_value(self.actual))


def _indent(s):
    return '\n'.join('  ' + line for line in s.splitlines())


def _format_value(value):
    return _indent(_generate_pprint_json(value))


def _generate_pprint_json(value):
    return json.dumps(value, sort_keys=True, indent=4)


def _is_dict_same(expected, actual, ignore_value_of_keys):
    # DAN - I had to flip flop this
    for key in expected:
        if key in ignore_value_of_keys:
            continue
        if key not in actual:
            return False, \
                   Stack().append(
                        StackItem('Expected key "{0}" Missing from Actual'
                                      .format(key),
                                  expected,
                                  actual))

        are_same_flag, stack = _are_same(expected[key], actual[key],ignore_value_of_keys)
        if not are_same_flag:
            return False, \
                   stack.append(StackItem('Different values', expected[key], actual[key]))
    return True, Stack()


def _is_list_same(expected, actual, ignore_value_of_keys):
    for i in xrange(len(expected)):
        are_same_flag, stack = _are_same(expected[i], actual[i], ignore_value_of_keys)
        if not are_same_flag:
            return False, \
                   stack.append(
                       StackItem('Different values (Check order)', expected[i], actual[i]))
    return True, Stack()


def _bottom_up_sort(unsorted_json):
    if isinstance(unsorted_json, list):
        new_list = []
        for i in xrange(len(unsorted_json)):
            new_list.append(_bottom_up_sort(unsorted_json[i]))
        return sorted(new_list)

    elif isinstance(unsorted_json, dict):
        new_dict = {}
        for key in sorted(unsorted_json):
            new_dict[key] = _bottom_up_sort(unsorted_json[key])
        return new_dict

    else:
        return unsorted_json


def _are_same(expected, actual, ignore_value_of_keys, ignore_missing_keys=False):
    # Check for None
    if expected is None:
        return expected == actual, Stack()

    # Ensure they are of same type
    if type(expected) != type(actual):
        return False, \
               Stack().append(
                   StackItem('Type Mismatch: Expected Type: {0}, Actual Type: {1}'
                                .format(type(expected), type(actual)),
                             expected,
                             actual))

    # Compare primitive types immediately
    if type(expected) in (int, str, bool, long, float, unicode):
        return expected == actual, Stack()

    if not ignore_missing_keys and len(expected) > len(actual):
        all_missing_keys = get_missing_keys(expected, actual)
        missing_keys = [x for x in all_missing_keys if x not in ignore_value_of_keys]
        if len(missing_keys):
            stack = Stack().append(StackItem('Length Mismatch: Expected Length: {0}, Actual Length: {1}'
                                              .format(len(expected), len(actual)), expected, actual))
            if isinstance(expected, dict):
              stack.append('\nMissing keys: {0}'.format(missing_keys))
              stack.append('\nIgnored keys: {0}'.format([x for x in ignore_value_of_keys if x not in all_missing_keys]))
            return False, stack

    if isinstance(expected, dict):
        return _is_dict_same(expected, actual, ignore_value_of_keys)

    if isinstance(expected, list):
        return _is_list_same(expected, actual, ignore_value_of_keys)

    return False, Stack().append(StackItem('Unhandled Type: {0}'.format(type(expected)), expected, actual))


def get_missing_keys(expected, actual):
    return [key for key in expected if key not in actual]


def are_same(original_a, original_b, ignore_list_order_recursively=False, ignore_missing_keys=False, ignore_value_of_keys=None):
    ignore_value_of_keys = ignore_value_of_keys if ignore_value_of_keys else []
    a = _bottom_up_sort(original_a) if ignore_list_order_recursively else original_a
    b = _bottom_up_sort(original_b) if ignore_list_order_recursively else original_b
    return _are_same(a, b, ignore_value_of_keys, ignore_missing_keys)


def contains(expected_original, actual_original, ignore_list_order_recursively=False, ignore_value_of_keys=None):
    ignore_value_of_keys = ignore_value_of_keys if ignore_value_of_keys else []
    actual = _bottom_up_sort(actual_original) if ignore_list_order_recursively else actual_original
    expected = _bottom_up_sort(expected_original) if ignore_list_order_recursively else expected_original
    return _are_same(expected, actual, ignore_value_of_keys, True)


def json_are_same(a, b, ignore_list_order_recursively=False, ignore_value_of_keys=None):
    ignore_value_of_keys = ignore_value_of_keys if ignore_value_of_keys else []
    return are_same(json.loads(a), json.loads(b), ignore_list_order_recursively, ignore_value_of_keys)




from xmljson import badgerfish as bf
import xml.etree.ElementTree as ET
import sys, xmljson
from json import dumps

tree = ET.parse(sys.argv[1])
print dumps(xmljson.yahoo.data(tree.getroot()),indent=2)

